/*
 * A very simple GNOME Shell extension that adds a "restart shell"
 * function to the user's menu, with a confirmation dialog.
 * Copyright (C) 2012  Andrea Santilli <andresantilli gmx com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

const Clutter       = imports.gi.Clutter;
const Gio           = imports.gi.Gio;
const GLib          = imports.gi.GLib;
const Lang          = imports.lang;
const Main          = imports.ui.main;
const ModalDialog   = imports.ui.modalDialog;
const PopupMenu     = imports.ui.popupMenu;
const St            = imports.gi.St;

const DEFAULT_STRINGS = {
    "entry-text" : "Restart Shell...",
    "cancel-btn" : "Cancel",
    "reload-btn" : "Reload theme only",
    "restart-btn" : "Restart",
    "reloaded-msg" : "Theme reloaded",
    "dialog-title" : "Restart the GNOME Shell",
    "dialog-description": "This action will restart the GNOME Shell.\n\
Do you really wish to continue?"
};

function ShellRestartMenuItem() {
    this._init.apply(this, arguments);
}

ShellRestartMenuItem.prototype = {
    __proto__: PopupMenu.PopupMenuItem.prototype,

    _init: function(metadata, params)
    {
        let statusMenu;
        let children;
        let index, i;

        PopupMenu.PopupMenuItem.prototype._init.call(this,
                strings['entry-text']);

        statusMenu = Main.panel._statusArea.userMenu;
        children = statusMenu.menu._getMenuItems();
        index = children.length;

        /* this item looks for the "lock screen" entry and puts itself right
         * above it, otherwise it goes to the bottom of the usermenu */
        for (i = index - 1; i >= 0; i--) {
            if (children[i] == statusMenu._lockScreenItem) {
                index = i;
                break;
            }
        }
        statusMenu.menu.addMenuItem(this, index);

        this.connect('activate', Lang.bind(this, function () {
            Main.overview.hide();
            new ShellRestartConfirmDialog().open();
        }));
    },

    destroy: function() {
        this.actor._delegate = null;
        this.actor.destroy();
        this.actor.emit('destroy');
    }
};

function ShellRestartConfirmDialog() {
    this._init.apply(this, arguments);
}

/* let's use the same layout of the logout dialog */
ShellRestartConfirmDialog.prototype = {
    __proto__: ModalDialog.ModalDialog.prototype,
    
    _init: function(metadata, params) {
        let contents, msgbox;
        let subject, description;

        ModalDialog.ModalDialog.prototype._init.call(this,
            { styleClass: 'end-session-dialog' });

        msgbox = new St.BoxLayout({ vertical: true });
        this.contentLayout.add(msgbox, { y_align: St.Align.START });

        subject = new St.Label({
            style_class: 'end-session-dialog-subject',
            text: strings['dialog-title']
        });
        msgbox.add(subject, { y_fill:  false, y_align: St.Align.START });

        description = new St.Label({
            style_class: 'end-session-dialog-description',
            text: strings['dialog-description']
        });
        msgbox.add(description, { y_fill:  true, y_align: St.Align.START });

        /* keys won't work in the dialog until bug #662493 gets fixed */
        this.setButtons([{
            label: strings['cancel-btn'],
            action: Lang.bind(this, function() {
                this.close();
            }),
            key: Clutter.Escape
        }, {
            label: strings['reload-btn'],
            action: Lang.bind(this, function() {
                this.close();
                Main.loadTheme();
                Main.notify(strings['reloaded-msg']);
            })
        }, {
            label: strings['restart-btn'],
            action: Lang.bind(this, function() {
                this.close();
                global.reexec_self();
            })
        }]);
    }
};

/* Using gettext in a GS script can be annoying for both scripters and users:
 * it forces the first ones to manage external repositories, while the latters
 * can't eventually modify compiled localizations. Also, users need to know
 * how to use gettext in order to modify and compile .po source files.
 * In my opinion the most convenient alternative is to use json files, as they
 * are easy to understand, modify and create for everyone. */
function localize_strings(ext_path) {
    let langs = GLib.get_language_names();
    let ret = false;
    let lc;

    /* for each locale identifier ID, check whether there's a file named
     * locale-ID.json, then try to read it and to parse its contents. */
    for (let i = 0; i < langs.length; i++) {
        let file = Gio.file_new_for_path(GLib.build_filenamev([
                ext_path, 'locale-' + langs[i] + '.json' ]));

        if (file.query_exists(null)) {
            let data;

            [ ret, data ] = file.load_contents(null);
            if (ret) {
                try {
                    lc = JSON.parse(data);
                } catch(e) {
                    global.log('Badly formatted JSON file: "' +
                            file.get_path() + '".');
                    ret = false;
                    continue;
                }
                //global.log('"' + file.get_path() + '" successfully parsed.');
                break;
            } else {
                global.log('Could not read "' + file.get_path() + '".');
            }
        }
    }

    /* file not found, not readable or not parsable: assign the default
     * strings (which are hardcoded in english) */
    if (!ret)
        lc = DEFAULT_STRINGS;
    else
        /* check whether all keys are associated to a string. If not,
         * assign them their respective default strings. This allows us to
         * use incomplete localizations. */
        for (let i = 0; i < Object.keys(DEFAULT_STRINGS).length; i++) {
            let key = Object.keys(DEFAULT_STRINGS)[i];
            if (lc[key] == undefined)
                lc[key] = DEFAULT_STRINGS[key];
        }

    return lc;
}

let srmi, strings, md;

function init(metadata) {
    md = metadata;
}

function enable(metadata) {
    /* we'd better do this here than in init() */
    strings = localize_strings(md.path);

    srmi = new ShellRestartMenuItem();
}

function disable() {
    /* the GC should guess right, here */
    strings = null;

    srmi.destroy();
}

